@@ -163,6 +163,9 @@ st_cast.sfc = function(x, to, ..., ids = seq_along(x), group_or_split = TRUE) {
 		reclass(ret, to, need_close(to))
 	} else if (from_col == 3 && to == "MULTILINESTRING") {
 		ret = lapply(x, unlist, recursive = FALSE) # unlist one level deeper; one MULTIPOLYGON -> one MULTILINESTRING
+		if (length(ret))
+			class(ret[[1]]) = class(x[[1]]) # got dropped
+		attributes(ret) = attributes(x) # bbox, crs
 		structure(reclass(ret, to, FALSE))
 	} else { # "horizontal", to the left: split
 		ret = if (from_col == 1) # LINESTRING or MULTIPOINT to POINT
