@@ -8,7 +8,7 @@
 #' @param nbreaks number of colors breaks (ignored for \code{factor} or \code{character} variables)
 #' @param breaks either a numeric vector with the actual breaks, or a name of a method accepted by the \code{style} argument of \link[classInt]{classIntervals}
 #' @param max.plot integer; lower boundary to maximum number of attributes to plot; the default value (9) can be overriden by setting the global option \code{sf_max.plot}, e.g. \code{options(sf_max.plot=2)}
-#' @param key.pos integer; which side to plot a color key: 1 bottom, 2 left, 3 top, 4 right. Set to \code{NULL} for no key. Currently ignored if multiple columns are plotted.
+#' @param key.pos integer; side to plot a color key: 1 bottom, 2 left, 3 top, 4 right; set to \code{NULL} to omit key. Ignored if multiple columns are plotted in a single function call. Default depends on plot size, map aspect, and, if set, parameter \code{asp}.
 #' @param key.size amount of space reserved for the key (labels)
 #' @param pch plotting symbol
 #' @param cex symbol size
@@ -19,6 +19,7 @@
 #' @param border color of polygon border
 #' @param add logical; add to current plot?
 #' @param type plot type: 'p' for points, 'l' for lines, 'b' for both
+#' @param reset logical; if \code{FALSE}, keep the plot in a mode that allows adding further map elements; if \code{TRUE} restore original mode after plotting; see details.
 #' @method plot sf
 #' @name plot
 #' @details \code{plot.sf} maximally plots \code{max.plot} maps with colors following from attribute columns,
@@ -28,6 +29,8 @@
 #' \code{plot.sfc} plots the geometry, additional parameters can be passed on
 #' to control color, lines or symbols.
 #'
+#' When setting \code{reset} to \code{FALSE}, the original device parameters are lost, and the device must be reset using \code{dev.off()} in order to reset it.
+#'
 #' @examples
 #' # plot linestrings:
 #' l1 = st_linestring(matrix(runif(6)-0.5,,2))
@@ -86,40 +89,46 @@
 #' gc = st_sf(a=2:3, b = st_sfc(gc1,gc2))
 #' plot(gc, cex = gc$a, col = gc$a, border = rev(gc$a) + 2, lwd = 2)
 #' @export
-plot.sf <- function(x, y, ..., col = NULL, main, pal = NULL, nbreaks = 10, breaks = "pretty", 
-		max.plot = if(is.null(n <- options("sf_max.plot")[[1]])) 9 else n, 
-		key.pos = if (ncol(x) > 2) NULL else 4, key.size = lcm(1.8)) {
+plot.sf <- function(x, y, ..., col = NULL, main, pal = NULL, nbreaks = 10, breaks = "pretty",
+		max.plot = if(is.null(n <- options("sf_max.plot")[[1]])) 9 else n,
+		key.pos = get_key_pos(x, ...), key.size = lcm(1.8), reset = TRUE) {
 
 	stopifnot(missing(y))
 	nbreaks.missing = missing(nbreaks)
 	key.pos.missing = missing(key.pos)
+	max_plot_missing = missing(max.plot)
 	dots = list(...)
 
+	opar = par()
 	if (ncol(x) > 2 && !isTRUE(dots$add)) { # multiple maps to plot...
-		max_plot_missing = missing(max.plot)
 		cols = setdiff(names(x), attr(x, "sf_column"))
-		mfrow = get_mfrow(st_bbox(x), min(max.plot, length(cols)), par("din"))
-		opar = if (isTRUE(dots$axes))
-				par(mfrow = mfrow, mar = c(2.1, 2.1, 1.2, 0))
-			else
-				par(mfrow = mfrow, mar = c(0,0,1.2,0))
-		on.exit(par(opar))
+		lt = .get_layout(st_bbox(x), min(max.plot, length(cols)), par("din"), NULL, key.size)
+		key.pos = lt$key.pos
+		layout(lt$m, widths = lt$widths, heights = lt$heights, respect = FALSE)
+
+		if (isTRUE(dots$axes))
+			par(mar = c(2.1, 2.1, 1.2, 0))
+		else
+			par(mar = c(0,0,1.2,0))
 
 		if (max_plot_missing)
-			max.plot = prod(mfrow)
+			max.plot = prod(lt$mfrow)
 
 		if (isTRUE(is.finite(max.plot)) && ncol(x) - 1 > max.plot) {
 			if (max_plot_missing && is.null(options("sf_max.plot")[[1]]))
 				warning(paste("plotting the first", max.plot, "out of", ncol(x)-1,
 					"attributes; use max.plot =", ncol(x) - 1, "to plot all"), call. = FALSE)
-			x = x[, 1:max.plot]
 		}
 		# col selection may have changed; set cols again:
 		cols = setdiff(names(x), attr(x, "sf_column"))
+		if (length(cols) > max.plot)
+			cols = cols[1:max.plot]
 		# loop over each map to plot:
 		invisible(lapply(cols, function(cname) plot(x[, cname], main = cname, col = col,
-			pal = pal, nbreaks = nbreaks, breaks = breaks, key.pos = NULL, ...)))
+			pal = pal, nbreaks = nbreaks, breaks = breaks, key.pos = NULL, reset = FALSE, ...)))
 	} else { # single map, or dots$add=TRUE:
+		if (!identical(TRUE, dots$add) && reset)
+			layout(matrix(1)) # reset
 		if (is.null(col) && ncol(x) == 1) # no colors, no attributes to choose colors from: plot geometry
 			plot(st_geometry(x), ...)
 		else { # generate plot with colors and possibly key
@@ -180,7 +189,7 @@ plot.sf <- function(x, y, ..., col = NULL, main, pal = NULL, nbreaks = 10, break
 								pal
 						colors[cuts]
 					}
-			} else { 
+			} else {
 				if (is.factor(values)) {
 					which.first = function(x) which(x)[1]
 					fnum = as.numeric(values)
@@ -190,8 +199,8 @@ plot.sf <- function(x, y, ..., col = NULL, main, pal = NULL, nbreaks = 10, break
 				} else # no key:
 					key.pos = NULL
 			}
-			
-			if (! is.null(key.pos) && !all(is.na(values)) &&
+
+			if (! isTRUE(dots$add) && ! is.null(key.pos) && !all(is.na(values)) &&
 					(is.factor(values) || length(unique(na.omit(values))) > 1) &&
 					length(col) > 1) { # plot key?
 				switch(key.pos,
@@ -201,10 +210,10 @@ plot.sf <- function(x, y, ..., col = NULL, main, pal = NULL, nbreaks = 10, break
 					layout(matrix(c(2,1), nrow = 1, ncol = 2), widths = c(1, key.size), heights = 1)   # 4 right
 				)
 				if (is.factor(values)) {
-					image.scale.factor(levels(values), colors, key.pos = key.pos, 
+					.image_scale_factor(levels(values), colors, key.pos = key.pos,
 						axes = isTRUE(dots$axes), key.size = key.size)
 				} else
-					image.scale(values, colors, breaks = breaks, key.pos = key.pos, axes = isTRUE(dots$axes))
+					.image_scale(values, colors, breaks = breaks, key.pos = key.pos, axes = isTRUE(dots$axes))
 			}
 			# plot the map:
 			mar = c(1, 1, 1.2, 1)
@@ -221,9 +230,34 @@ plot.sf <- function(x, y, ..., col = NULL, main, pal = NULL, nbreaks = 10, break
 			}
 			title(main)
 		}
+		if (reset) {
+			layout(matrix(1)) # reset
+			desel = which(names(opar) %in% c("cin", "cra", "csi", "cxy", "din", "page", "fig"))
+			par(opar[-desel])
+		}
 	}
 }
 
+#' @name plot
+#' @export
+get_key_pos = function(x, ...) {
+	bb = st_bbox(x)
+	if (any(is.na(bb)) || (inherits(x, "sf") && ncol(x) > 2))
+		NULL
+	else {
+		pin = par("pin") # (width, height)
+		asp_plt = pin[2]/pin[1] # y/x: < 1 means wide
+		asp_box = diff(bb[c(4,2)]) / diff(bb[c(3,1)])
+		asp = list(...)$asp
+		if (is.null(asp))
+			asp <- ifelse(isTRUE(st_is_longlat(x)), 1/cos((mean(bb[c(2,4)]) * pi)/180), 1.0)
+		asp_box = asp_box * asp
+		if (!is.finite(asp_box) || asp_box < asp_plt) # wider
+			1
+		else # taller
+			4
+	}
+}
 
 #' @name plot
 #' @method plot sfc_POINT
@@ -330,7 +364,7 @@ p_bind = function(lst) {
 }
 
 #' @name plot
-#' @param rule see \link[graphics]{polypath}; for \code{winding}, exterior ring direction should be opposite that of the holes; with \code{evenodd}, plotting is robust against misspecified ring directions 
+#' @param rule see \link[graphics]{polypath}; for \code{winding}, exterior ring direction should be opposite that of the holes; with \code{evenodd}, plotting is robust against misspecified ring directions
 #' @export
 plot.sfc_POLYGON = function(x, y, ..., lty = 1, lwd = 1, col = NA, cex = 1, pch = NA, border = 1,
 		add = FALSE, rule = "evenodd") {
@@ -511,7 +545,7 @@ plot_sf = function(x, xlim = NULL, ylim = NULL, asp = NA, axes = FALSE, bgc = pa
 	pl_reg <- par("usr")
 	rect(xleft = pl_reg[1], ybottom = pl_reg[3], xright = pl_reg[2],
 		ytop = pl_reg[4], col = bgc, border = FALSE)
-	linAxis = function(side, ..., lon, lat, ndiscr) axis(side = side, ...)
+	linAxis = function(side, ..., lon, lat, ndiscr, reset) axis(side = side, ...)
 	if (! missing(graticule)) {
 		g = if (isTRUE(graticule))
 				st_graticule(pl_reg[c(1,3,2,4)], st_crs(x), st_crs(4326), ...)
@@ -530,14 +564,14 @@ plot_sf = function(x, xlim = NULL, ylim = NULL, asp = NA, axes = FALSE, bgc = pa
 	} else if (axes) {
 		box()
 		if (isTRUE(st_is_longlat(x))) {
-			degAxis(1, ...)
-			degAxis(2, ...)
+			.degAxis(1, ...)
+			.degAxis(2, ...)
 		} else {
 			linAxis(1, ...)
 			linAxis(2, ...)
 		}
 	}
-	localTitle <- function(..., col, bgc, pch, cex, lty, lwd, lon, lat, ndiscr, at, labels) title(...)
+	localTitle <- function(..., col, bgc, pch, cex, lty, lwd, lon, lat, ndiscr, at, labels, reset) title(...)
 	localTitle(...)
 	if (!is.null(bgMap)) {
 		mercator = FALSE
@@ -592,24 +626,61 @@ sf.colors = function (n = 10, cutoff.tails = c(0.35, 0.2), alpha = 1, categorica
 	}
 }
 
-get_mfrow = function(bb, n, total_size = c(1,1)) {
-	asp = diff(bb[c(1,3)])/diff(bb[c(2,4)])
+#' @export
+#' @name stars
+#' @param bb ignore
+#' @param n ignore
+#' @param total_size ignore
+.get_layout = function(bb, n, total_size, key.pos, key.size) {
+# return list with "m" matrix, "key.pos", "widths" and "heights" fields
+# if key.pos = -1, it will be a return value, "optimally" placed
+	asp = diff(bb[c(2,4)])/diff(bb[c(1,3)])
+	if (isTRUE(st_is_longlat(bb)))
+		asp = asp / cos(mean(bb[c(2,4)]) * pi /180)
 	size = function(nrow, n, asp) {
 		ncol = ceiling(n / nrow)
 		xsize = total_size[1] / ncol
-		ysize = xsize  / asp
+		ysize = xsize  * asp
 		if (xsize * ysize * n > prod(total_size)) {
 			ysize = total_size[2] / nrow
-			xsize = ysize * asp
+			xsize = ysize / asp
 		}
 		xsize * ysize
 	}
 	sz = vapply(1:n, function(x) size(x, n, asp), 0.0)
 	nrow = which.max(sz)
 	ncol = ceiling(n / nrow)
-	structure(c(nrow, ncol), names = c("nrow", "ncol"))
-}
 
+	ret = list()
+	ret$mfrow = c(nrow, ncol)
+
+	# the following is right now only used by stars; FIXME:
+	# nocov start
+	ret$key.pos = if (!is.null(key.pos) && key.pos == -1L) { # figure out here: right or bottom?
+			newasp = asp * ncol / nrow # of the composition
+			dispasp = total_size[1] / total_size[2]
+			ifelse(newasp > dispasp, 1, 4) # > or < ? oh dear,
+		} else
+			key.pos
+
+	m = matrix(1 : (nrow * ncol), nrow, ncol, byrow = TRUE)
+	if (!is.null(ret$key.pos) && ret$key.pos != 0) {
+		k = key.size
+		n = nrow * ncol + 1
+		switch(ret$key.pos,
+			{ ret$m = rbind(m, n); ret$widths = c(rep(1, ncol)); ret$heights = c(rep(1, nrow), k) },
+			{ ret$m = cbind(n, m); ret$widths = c(k, rep(1, ncol)); ret$heights = c(rep(1, nrow)) },
+			{ ret$m = rbind(n, m); ret$widths = c(rep(1, ncol)); ret$heights = c(k, rep(1, nrow)) },
+			{ ret$m = cbind(m, n); ret$widths = c(rep(1, ncol), k); ret$heights = c(rep(1, nrow)) }
+		)
+	} else {
+		ret$m = m
+		ret$widths = rep(1, ncol)
+		ret$heights = rep(1, nrow)
+	}
+	# nocov end
+	ret
+}
 
 bb2merc = function(x, cls = "ggmap") { # return bbox in the appropriate "web mercator" CRS
 	wgs84 = st_crs(4326)
@@ -624,7 +695,17 @@ bb2merc = function(x, cls = "ggmap") { # return bbox in the appropriate "web mer
 	st_bbox(st_transform(pts, merc))
 }
 
-degAxis = function (side, at, labels, ..., lon, lat, ndiscr) {
+#' functions only exported to be used internally by stars
+#' @name stars
+#' @export
+#' @param side ignore
+#' @param at ignore
+#' @param labels ignore
+#' @param lon ignore
+#' @param lat ignore
+#' @param ndiscr ignore
+#' @param reset ignore
+.degAxis = function (side, at, labels, ..., lon, lat, ndiscr, reset) {
 	if (missing(at))
        	at = axTicks(side)
 	if (missing(labels)) {
@@ -637,7 +718,16 @@ degAxis = function (side, at, labels, ..., lon, lat, ndiscr) {
 	axis(side, at = at, labels = labels, ...)
 }
 
-image.scale = function(z, col, breaks = NULL, key.pos, add.axis = TRUE,
+#' @name stars
+#' @export
+#' @param z ignore
+#' @param col ignore
+#' @param breaks ignore
+#' @param key.pos ignore
+#' @param add.axis ignore
+#' @param axes ignore
+#' @param ... ignore
+.image_scale = function(z, col, breaks = NULL, key.pos, add.axis = TRUE,
 	at = NULL, ..., axes = FALSE) {
 	if (!is.null(breaks) && length(breaks) != (length(col) + 1))
 		stop("must have one more break than colour")
@@ -662,19 +752,34 @@ image.scale = function(z, col, breaks = NULL, key.pos, add.axis = TRUE,
 		poly[[i]] = c(breaks[i], breaks[i+1], breaks[i+1], breaks[i])
 	plot(1, 1, t = "n", ylim = ylim, xlim = xlim, axes = FALSE,
 		xlab = "", ylab = "", xaxs = "i", yaxs = "i", ...)
+	offset = 0.2
+	offs = switch(key.pos,
+		c(0,0,-offset,-offset),
+		c(0,0,-offset,-offset),
+		c(offset,offset,0,0),
+		c(offset,offset,0,0)) 
 	for(i in seq_along(poly)) {
 		if (key.pos %in% c(1,3))
-			polygon(poly[[i]], c(0, 0, 1, 1), col=col[i], border=NA)
+			polygon(poly[[i]], c(0, 0, 1, 1) + offs, col=col[i], border=NA)
 		if (key.pos %in% c(2,4))
-			polygon(c(0, 0, 1, 1), poly[[i]], col=col[i], border=NA)
+			polygon(c(0, 0, 1, 1) + offs, poly[[i]], col=col[i], border=NA)
 	}
 
-	box()
+	# box() now would draw around [0,1]:
+	bx = c(breaks[1], rep(tail(breaks, 1), 2), breaks[1])
+	if (key.pos %in% c(1,3))
+		polygon(bx, c(0, 0, 1, 1) + offs, col = NA, border = 'black')
+	if (key.pos %in% c(2,4))
+		polygon(c(0, 0, 1, 1) + offs, bx, col = NA, border = 'black')
+
 	if (add.axis)
 		axis(key.pos, at)
 }
 
-image.scale.factor = function(z, col, breaks = NULL, key.pos, add.axis = TRUE,
+#' @name stars
+#' @export
+#' @param key.size ignore
+.image_scale_factor = function(z, col, breaks = NULL, key.pos, add.axis = TRUE,
 	at = NULL, ..., axes = FALSE, key.size) {
 
 	n = length(z)
@@ -719,8 +824,6 @@ image.scale.factor = function(z, col, breaks = NULL, key.pos, add.axis = TRUE,
 identify.sfc = function(x, ..., n = min(10, length(x)), type = "n") {
 	l = locator(n, type = type)
 	pts = st_as_sf(as.data.frame(do.call(cbind, l)), coords = c("x", "y"), crs = st_crs(x))
-	#i = st_intersects(x, pts) 
-	#which(lengths(i) > 0)
 	sapply(st_intersects(pts, x), function(x) if (length(x)) x[1] else NA_integer_)
 }
 
