@@ -7,8 +7,12 @@
 #' @param ... ignored
 #' @param partial logical; allow for partial projection, if not all points of a geometry can be projected (corresponds to setting environment variable \code{OGR_ENABLE_PARTIAL_REPROJECTION} to \code{TRUE})
 #' @param check logical; perform a sanity check on resulting polygons?
-#' @param use_gdal logical; this parameter is deprecated. Use \link[lwgeom]{st_transform_proj} instead
+#' @param use_gdal logical; this parameter is deprecated. For transformations using PROJ.4 directly rather than indirectly through GDAL, use \link[lwgeom]{st_transform_proj} of package \code{lwgeom} (see Details)
 #' @details Transforms coordinates of object to new projection. Features that cannot be transformed are returned as empty geometries.
+#' 
+#' \code{st_transform} uses GDAL for coordinate transformations; internally, GDAL converts the \code{proj4string} into a well-known-text representation, before passing that on to PROJ.4. In this process, some information can get lost. Adding parameter \code{+wktext} to the \code{proj4string} definition may resolve this; see \url{https://github.com/edzer/sp/issues/42}.
+#' 
+#' Some PROJ.4 projections are not supported by GDAL, e.g. \code{"+proj=wintri"} because it does not have an inverse projection. Projecting to unsupported projections can be done by \link[lwgeom]{st_transform_proj}, part of package lwgeom. Note that the unsupported \code{proj4string} cannot be passed as argument to \link{st_crs}, but has to be given as character string.
 #' @examples
 #' p1 = st_point(c(7,52))
 #' p2 = st_point(c(-30,20))
@@ -20,8 +24,10 @@ st_transform = function(x, crs, ...) UseMethod("st_transform")
 
 chk_pol = function(x, dim = class(x)[1]) {
 	PolClose = function(y) {
-		if (any(head(y[[1]], 1) != tail(y[[1]], 1)))
+		if (any(head(y[[1]], 1) != tail(y[[1]], 1))) # close
 			y[[1]] = rbind(y[[1]], head(y[[1]], 1))
+		else if (nrow(y[[1]]) == 3) # closed, but line
+			return(st_polygon(dim = dim))
 		y
 	}
 	if (length(x) > 0 && nrow(x[[1]]) > 2)
@@ -125,6 +131,10 @@ st_transform.sfg = function(x, crs , ...) {
 #' @examples
 #' st_proj_info("datum")
 st_proj_info = function(type = "proj") {
+
+	if (type == "have_datum_files")
+		return(CPL_have_datum_files(0))
+
     opts <- c("proj", "ellps", "datum", "units")
     if (!(type %in% opts)) stop("unknown type")
     t <- as.integer(match(type[1], opts) - 1)
@@ -185,5 +195,7 @@ st_to_s2 = function(x) {
 #' @param pts two-column numeric matrix, or object that can be coerced into a matrix
 #' @export
 sf_project = function(from, to, pts) {
+	#.Deprecated("lwgeom::st_transform_proj")
 	CPL_proj_direct(as.character(c(from[1], to[1])), as.matrix(pts))
 }
+
