@@ -15,3 +15,11 @@ c(st_geometrycollection(list(st_point(1:2), st_linestring(matrix(1:6,3)))),
   st_multilinestring(list(matrix(11:16,3))), st_point(5:6), 
   st_geometrycollection(list(st_point(10:11))))
 head(st_point(0:1), 2)
+
+# Ops.sfg:
+ls = st_linestring(rbind(c(0,0),c(0,1)))
+pt = st_point(1:0)
+ls + pt
+ls - pt
+ls * pt
+ls / pt
