@@ -4,9 +4,7 @@ test_that("st_crs works", {
   nc1 = st_read(system.file("shape/nc.shp", package="sf"), "nc", crs = 4267, quiet = TRUE)
   nc2 = st_read(system.file("shape/nc.shp", package="sf"), "nc", quiet = TRUE)
   nc3 = st_read(system.file("shape/nc.shp", package="sf"), "nc", crs = NA, quiet = TRUE)
-  crs_4267 <- structure(
-    list(epsg = 4267L, proj4string = "+proj=longlat +datum=NAD27 +no_defs"), 
-    class = "crs")
+  crs_4267 <- st_crs(4267)
   expect_equal(st_crs(nc2), crs_4267)
   expect_equal(st_crs(nc3), NA_crs_)
   expect_equal(st_set_crs(nc3, 4267) %>% st_crs, crs_4267)
@@ -37,6 +35,7 @@ test_that("st_proj_info works", {
   expect_silent(x <- st_proj_info("ellps"))
   expect_silent(x <- st_proj_info("datum"))
   expect_silent(x <- st_proj_info("units"))
+  expect_silent(x <- st_proj_info("have_datum_files"))
 })
 
 test_that("$.crs works", {
